import EventDispatcher from "./EventDispatcher.js";
import { queryKey } from "./Utils.js";

/**
 * @class Query
 */
export default class Query {
  /**
   * @param {Array(Component)} Components List of types of components to query
   */
  constructor(Components, manager) {
    this.Components = Components;
    this.entities = [];
    this.eventDispatcher = new EventDispatcher();

    // This query is being used by a ReactiveSystem
    this.reactive = false;

    this.key = queryKey(Components);

    // Fill the query with the existing entities
    for (var i = 0; i < manager._entities.length; i++) {
      var entity = manager._entities[i];
      if (entity.hasAllComponents(Components)) {
        this.entities.push(entity);
      }
    }
  }

  /**
   * Return stats for this query
   */
  stats() {
    return {
      numComponents: this.Components.length,
      numEntities: this.entities.length
    };
  }
}

Query.prototype.ENTITY_ADDED = "Query#ENTITY_ADDED";
Query.prototype.ENTITY_REMOVED = "Query#ENTITY_REMOVED";
Query.prototype.COMPONENT_CHANGED = "Query#COMPONENT_CHANGED";
