import Query from "./Query.js";

/**
 * @class ReactiveSystem
 */
export class ReactiveSystem {
  constructor(world, attributes) {
    this.world = world;
    this.enabled = true;
    this.queryComponents = this.init ? this.init() : null;
    this._queries = {};
    this.queries = {};
    this.priority = 0;

    if (attributes) {
      if (attributes.priority) { this.priority = attributes.priority; }
    }

    this.counters = {
      added: 0,
      removed: 0,
      changed: 0,
      componentChanged: 0
    };

    for (var name in this.queryComponents) {
      var Components = this.queryComponents[name];
      var query = this.world.entityManager.queryComponents(Components);
      this._queries[name] = query;
      this.queries[name] = {};
      var reactive = false;

      if (this.onEntitiesAdded) {
        reactive = true;
        this.queries[name].added = [];
        query.eventDispatcher.addEventListener(
          Query.prototype.ENTITY_ADDED,
          entity => {
            this.queries[name].added.push(entity);
            this.counters.added++;
          }
        );
      }

      if (this.onEntitiesRemoved) {
        reactive = true;
        this.queries[name].removed = [];
        query.eventDispatcher.addEventListener(
          Query.prototype.ENTITY_REMOVED,
          entity => {
            this.queries[name].removed.push(entity);
            this.counters.removed++;
          }
        );
      }

      if (this.onEntitiesChanged) {
        reactive = true;
        this.queries[name].changed = [];
        query.eventDispatcher.addEventListener(
          Query.prototype.COMPONENT_CHANGED,
          entity => {
            this.queries[name].changed.push(entity);
            this.counters.changed++;
          }
        );
      }

      query.reactive = reactive;

/*
      @todo
      if (this.onComponentChanged) {
        this.queries[name].componentChanged = [];
        query.eventDispatcher.addEventListener(
          Query.prototype.COMPONENT_CHANGED,
          entity => {
            this.queries[name].componentChanged.push({entity: entity, component: component});
            this.counters.componentChanged++;
          }
        );
      }
*/
    }
  }

  clearQueries() {
    for (var name in this.queries) {
      let query = this.queries[name];
      for (var event in query) {
        query[event].length = 0;
      }
    }
    this.counters.added = this.counters.removed = this.counters.changed = this.counters.componentChanged = 0;
  }
}
